#include "parser.h"
#include "shift_register.h"
#include "adc.h"
#include "relais.h"
#include "spi.h"
#include "uart.h"
#include "pattern.h"


#include <avr/io.h>
#include <avr/interrupt.h>

//a list of all commands used
#define		CMD_A2D				'a'	//sample analog data
#define		CMD_FREQ			'f' //set frequency
#define		CMD_OUT				'o'	//set digital output
#define		CMD_IN				'i'	//read digital input
#define		CMD_PATTERN			'p'	//set pattern
#define		CMD_SAMPLE			's'	//sample digital data
#define		CMD_LOGIC_OP	 	'x'	//set pattern and sample
#define		CMD_RELAIS		 	'r'	//set the relais state
#define		CMD_SPI_TRANSFER 	't'	//transfer spi data
#define		CMD_SPI_CONFIG		'c'	//config the spi bus
#define		CMD_SET_TRIGGER		'z'	//set a trigger for the next pattern/sample operation
#define		CMD_RESET			'y'
#define		CMD_SET_SERVO		'm'
#define		CMD_SOFTWARE_VER	'v'
#define		CMD_NO_COMMAND		0


uint8_t cmd;

volatile uint8_t n_samples;

void 	Parser_Init(void)
{
	cmd = CMD_NO_COMMAND;
	n_samples = 0;
}

void	Parser_Process(void)
{
	//if no command is beeing processed, save next byte
	if( (cmd == CMD_NO_COMMAND) && Uart_DataAvaible(1) )
	{
		cmd = Uart_GetByte();
	}

	//process command
	switch(cmd)
	{
		case CMD_A2D:
		{
			if(Uart_DataAvaible(1)) //we need a channel (1 byte)
			{
				uint8_t adc = Uart_GetByte();

				//Sample data and send it
				if( adc&0x01 )
					Uart_SendByte( ADC_Read(0) );
				if( adc&0x02 )
					Uart_SendByte( ADC_Read(1) );
				if( adc&0x04 )
					Uart_SendByte( ADC_Read(2) );
				if( adc&0x08 )
					Uart_SendByte( ADC_Read(3) );
				cmd = CMD_NO_COMMAND;
			}
		}break;
		case CMD_FREQ:
		{
			if(Uart_DataAvaible(3))
			{
				uint8_t prescaler = Uart_GetByte();
				uint8_t comp_high_byte = Uart_GetByte();
				uint8_t comp_low_byte = Uart_GetByte();

				Pattern_SetTimerParameter( prescaler, (uint16_t)comp_low_byte | ((uint16_t)comp_high_byte <<8 ) );
				cmd = CMD_NO_COMMAND;
			}
		}break;
		case CMD_OUT:
		{
			if(Uart_DataAvaible(2))
			{
				uint8_t high_byte = Uart_GetByte();
				uint8_t low_byte = Uart_GetByte();

				Pattern_SetOutput( (uint16_t)low_byte | ((uint16_t)high_byte <<8 ) );

				cmd = CMD_NO_COMMAND;
			}
		}break;
		case CMD_IN:
		{
			uint16_t in = Pattern_GetInput();

			Uart_SendByte(in>>8);
			Uart_SendByte(in&0xFF);

			cmd = CMD_NO_COMMAND;
		}break;
		case CMD_PATTERN:
		{
			if( Uart_DataAvaible(1) )
			{
				if(n_samples == 0)	//save number of samples
				{
					n_samples = Uart_GetByte();
					Pattern_Pattern(n_samples);
				}
			}

			if(Uart_DataAvaible(2) && n_samples)
			{
				uint8_t high_byte = Uart_GetByte();
				uint8_t low_byte = Uart_GetByte();

				Pattern_AddOutputData( (uint16_t)high_byte << 8 | low_byte);

				n_samples--;
				if(n_samples == 0)
					cmd = CMD_NO_COMMAND;
			}
		}break;
		case CMD_SAMPLE:
		{
			if(n_samples == 0 && Uart_DataAvaible(1))	//save number of samples
			{
				n_samples = Uart_GetByte();
				Pattern_Sample(n_samples);
			}

			if(n_samples)
			{
				uint16_t data;

				if(Pattern_GetInputData(&data) && Uart_TransferPossible(2))
				{
					Uart_SendByte( data >> 8 );
					Uart_SendByte( data & 0xFF );

					n_samples--;
					if(n_samples==0)
						cmd = CMD_NO_COMMAND;
				}
			}
		}break;
		case CMD_LOGIC_OP:
		{
			static uint8_t n_input_samples = 0;

			if(n_input_samples == 0 && n_samples == 0 && Uart_DataAvaible(1))	//save number of samples
			{
				n_input_samples = n_samples = Uart_GetByte();
				Pattern_LogicOp(n_samples);
				return;
			}
			else if(n_input_samples != 0 && n_samples != 0 && Uart_DataAvaible(2))
			{
				uint8_t high_byte = Uart_GetByte();
				uint8_t low_byte = Uart_GetByte();

				Pattern_AddOutputData( (uint16_t)high_byte << 8 | low_byte);

				n_samples--;
			}
			else if(n_input_samples != 0 && n_samples == 0)
			{
				uint16_t data;

				if(Pattern_GetInputData(&data) && Uart_TransferPossible(2))
				{
					Uart_SendByte( data >> 8 );
					Uart_SendByte( data & 0xFF );

					n_input_samples--;
					if(n_input_samples == 0)
						cmd = CMD_NO_COMMAND;
				}
			}
		}break;
		case CMD_RELAIS:
		{
			if(Uart_DataAvaible(1))
			{
				uint8_t r = Uart_GetByte();

				Relais_Set(r);

				cmd = CMD_NO_COMMAND;
			}
		}break;
		case CMD_SPI_TRANSFER:
		{
			if(Uart_DataAvaible(1))
			{
				uint8_t tx_spi = Uart_GetByte();
				uint8_t rx_spi = SPI_Transmit(tx_spi);

				Uart_SendByte( rx_spi );
				cmd = CMD_NO_COMMAND;
			}
		}break;
		case CMD_SPI_CONFIG:
		{
			if(Uart_DataAvaible(2))
			{
				uint8_t spcr = Uart_GetByte();
				uint8_t spsr = Uart_GetByte();

				SPI_Config(spcr, spsr);

				cmd = CMD_NO_COMMAND;
			}
		}break;
		case CMD_SET_TRIGGER:
		{
			if(Uart_DataAvaible(4))
			{
				uint16_t trigger_1, trigger_2;

				uint8_t high_byte = Uart_GetByte();
				uint8_t low_byte = Uart_GetByte();

				trigger_1 = (uint16_t)low_byte | ((uint16_t)high_byte <<8 );

				high_byte = Uart_GetByte();
				low_byte = Uart_GetByte();

				trigger_2 = (uint16_t)low_byte | ((uint16_t)high_byte <<8 );

				Pattern_SetTrigger(trigger_1, trigger_2);

				cmd = CMD_NO_COMMAND;
			}
		}break;
		case CMD_RESET:
		{
			while(1) //the watchdog handles the rest
				asm volatile("nop");
		}break;
		case CMD_SET_SERVO:
		{
			if(Uart_DataAvaible(16))
			{
				uint16_t ocrs[8];
				for(uint8_t n=0;n<8;n++)
				{
					uint8_t high_byte = Uart_GetByte();
					uint8_t low_byte = Uart_GetByte();

					ocrs[n] = (uint16_t)low_byte | ((uint16_t)high_byte <<8 );
				}
				Pattern_SetServo(ocrs);

				cmd = CMD_NO_COMMAND;
			}
		}break;
		case CMD_SOFTWARE_VER:
		{
			Uart_SendByte('2');
			Uart_SendByte('.');
			Uart_SendByte('0');
			Uart_SendByte('0');
			cmd = CMD_NO_COMMAND;
		}break;
		case CMD_NO_COMMAND:
		{
			//wait
		}break;
		default:
		{
			cmd = CMD_NO_COMMAND;
		}break;
	}
}
